/* Use qsort to sort an array of pointers. This does some pointer writes in
   non-HeapSafe-compiled code, but doesn't corrupt reference counts as it
   just performs a permutation of pointers within a single object.

   Note that the debug library has to do special magic to support this, though
   (as it's tracking pointers more accurately).

   This version makes sure we handle hash-table-based reference sets correctly
   with the debug library.
*/
#include <heapsafe.h>
#include <stdlib.h>
#include <stdio.h>

int pint_compare(const void *p1, const void *p2)
{
  int *const *i1 = p1, *const *i2 = p2;

  return **i1 - **i2;
}

enum { SIZE = 256 };

int main(int argc, char **argv)
{
  int **numbers = hs_malloc(2 * SIZE * sizeof(int *));
  int *special;
  int i;

  for (i = 0; i < SIZE; i++)
    {
      numbers[i] = hs_malloc(sizeof(int));
      *numbers[i] = rand();
    }
  special = hs_malloc(sizeof(int));
  *special = rand();
  for (i = 0; i < SIZE; i++)
    numbers[i + SIZE] = special;

  qsort(numbers, 2 * SIZE, sizeof(int *), pint_compare);

  hs_delayed_free_start();
  for (i = 0; i < 2 * SIZE; i++)
    {
      printf("%d ", *numbers[i]);
      if (numbers[i] != special)
	HS_FREE(numbers[i]);
    }
  printf("\n");
  HS_FREE(special);
  HS_FREE(numbers);
  hs_delayed_free_end();

  return 0;
}
